<?php
// $Id: salsa_groups.admin.inc,v 1.1.2.3 2011/02/23 01:55:47 jcnventura Exp $

/**
 * @file
 * Contains the administrative functions of the salsa_groups module.
 *
 * This file is included by the salsa_groups module, and includes the
 * settings form.
 */

/**
 * Menu callback for the Salsa module settings form.
 *
 * @ingroup forms
 */
function salsa_groups_settings() {
  $last_refresh = variable_get('salsa_groups_refresh', 0);

  $form['groups_refresh'] = array(
    '#type' => 'fieldset',
    '#title' => t('Groups data'),
  );
  $form['groups_refresh']['last'] = array(
    '#value' => '<div>'. ($last_refresh ? t('Group info was last downloaded on !date.', array('!date' => format_date($last_refresh, 'medium'))) : t('Group info was never downloaded from Salsa.')) .'</div>',
  );
  $form['groups_refresh']['refresh'] = array(
    '#type' => 'submit',
    '#value' => t('Refresh data'),
    '#submit' => array('salsa_groups_refresh'),
  );

  // Retrieve role names for columns.
  $role_names = user_roles();

  // Render role/permission overview:
  $options = array();

  $groups = db_query("SELECT groups_KEY, Group_Name FROM {salsa_groups}");
  while ($group = db_fetch_object($groups)) {
    $form['group'][$group->groups_KEY] = array(
      '#value' => $group->Group_Name,
    );
    $options[$group->groups_KEY] = '';
    $rids = db_query("SELECT rid FROM {salsa_groups_roles} WHERE groups_KEY = %d", $group->groups_KEY);
    while ($rid = db_fetch_object($rids)) {
      // Builds arrays for checked boxes for each role
      $status[$rid->rid][] = $group->groups_KEY;
    }
  }

  // Have to build checkboxes here after checkbox arrays are built
  foreach ($role_names as $rid => $name) {
    $form['checkboxes'][$rid] = array('#type' => 'checkboxes', '#options' => $options, '#default_value' => isset($status[$rid]) ? $status[$rid] : array());
    $form['role_names'][$rid] = array('#value' => $name, '#tree' => TRUE);
  }

  $form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));

  return $form;
}

/**
 * Theme the groups page.
 *
 * @ingroup themeable
 */
function theme_salsa_groups_settings($form) {
  $roles = user_roles();
  foreach (element_children($form['group']) as $key) {
    // Don't take form control structures
    if (is_array($form['group'][$key])) {
      $row = array();
      $row[] = array('data' => drupal_render($form['group'][$key]), 'class' => 'group');
      foreach (element_children($form['checkboxes']) as $rid) {
        if (is_array($form['checkboxes'][$rid])) {
          $row[] = array('data' => drupal_render($form['checkboxes'][$rid][$key]), 'class' => 'checkbox', 'title' => $roles[$rid] .' : '. t($key));
        }
      }
      $rows[] = $row;
    }
  }
  $header[] = (t('Permission'));
  foreach (element_children($form['role_names']) as $rid) {
    if (is_array($form['role_names'][$rid])) {
      $header[] = array('data' => drupal_render($form['role_names'][$rid]), 'class' => 'checkbox');
    }
  }

  $buttons = drupal_render($form['buttons']);

  $output = drupal_render($form);
  $output .= theme('table', $header, $rows, array('id' => 'permissions'));
  $output .= $buttons;

  return $output;
}

function salsa_groups_settings_submit($form, &$form_state) {
  if ($form_state['clicked_button']['#name'] != 'reset') {
    foreach ($form_state['values'] as $rid => $groups) {
      if (is_array($groups)) {
        foreach ($groups as $group_key => $active) {
          if ($active != 0) {
            @db_query("INSERT INTO {salsa_groups_roles} (groups_KEY, rid) VALUES (%d, %d)", $group_key, $rid);
          }
          else {
            @db_query("DELETE FROM {salsa_groups_roles} WHERE groups_KEY = %d AND rid = %d", $group_key, $rid);
          }
        }
      }
    }
  }
  else {
    variable_del('salsa_groups_refresh');
    db_query('TRUNCATE TABLE {salsa_groups}');
    db_query('TRUNCATE TABLE {salsa_groups_roles}');
  }
}

/**
 * Refresh group information
 */
function salsa_groups_refresh() {
  $groups = salsa_get_objects('groups');
  variable_set('salsa_groups_refresh', time());

  db_query('TRUNCATE TABLE {salsa_groups}');

  if ($groups) {
    foreach ($groups->xpath('//item') as $group) {
      db_query("INSERT INTO {salsa_groups} (groups_KEY, organization_KEY, chapter_KEY, Group_Name, parent_KEY, Description, Display_To_User, Subscription_Type)
                  VALUES (%d, %d, %d, '%s', %d, '%s', %d, '%s')", (string)$group->groups_KEY, (string)$group->organization_KEY, (string)$group->chapter_KEY,
                  (string)$group->Group_Name, (string)$group->parent_KEY, (string)$group->Description, (string)$group->Display_To_User == 'true' ? 1 : 0,
                  (string)$group->Subscription_Type);
    }
  }
}
