<?php
// $Id: salsa_objects.admin.inc,v 1.1.2.9 2011/02/22 20:35:54 jcnventura Exp $

/**
 * @file
 * Contains the administrative functions of the salsa_objects module.
 *
 * This file is included by the salsa_objects module, and includes the
 * settings form.
 */

/**
 * Menu callback for the Salsa Supporter module settings form.
 *
 * @ingroup forms
 */
function salsa_objects_mappings() {
  $config = variable_get('salsa_objects_mappings', array());

  $object_fields = salsa_objects_table_fields();
  $options = array('' => '');
  foreach ($object_fields as $field => $value) {
    $options[$field] = drupal_ucfirst(str_replace('_', ' ', $field));
  }

  // If the flag module exists, allow it to be used for creating an object association
  $flag_options = array();
  if (module_exists('flag')) {
    $flags_config = variable_get('salsa_objects_flags', array());

    $flags = flag_get_flags();
    $flag_options[] = '';
    foreach ($flags as $flag_name => $flag) {
      $flag_options[$flag_name] = $flag->title;
    }
  }

  $form['salsa_objects_conflict'] = array(
    '#type' => 'select',
    '#title' => t('In case of conflict'),
    '#options' => array(
      'drupal' => t('Drupal wins'),
      'salsa' => t('Salsa wins'),
    ),
    '#default_value' => variable_get('salsa_objects_conflict', 'drupal'),
  );
  if (module_exists('content')) {
    $types = content_types();
    $field_types = _content_field_types();
  }
  else {
    $types = (array) node_get_types();
    $field_types = array('text' => array('label' => 'Text'));
  }

  foreach ($types as $type_id => $type) {
    $type = (array) $type;
    if (module_exists('content_profile') && is_content_profile($type['type'])) {
      // Skip any content type that is a content profile
      continue;
    }
    $form[$type_id] = array(
      '#type' => 'fieldset',
      '#title' => $type['name'] .' ('. t('Content type') .')',
      '#tree' => TRUE,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );

    // Fields.
    $fields = array();
    if ($type['has_title']) {
      $fields['title']['widget']['label'] = $type['title_label'];
      $fields['title']['type'] = 'text';
    }
    if ($type['has_body']) {
      $fields['body_field']['widget']['label'] = $type['body_label'];
      $fields['body_field']['type'] = 'text';
    }
    // Add CCK's fields
    if (isset($type['fields'])) {
      $fields = $fields + $type['fields'];
    }

    foreach ($fields as $name => $field) {
      $config_value = $config[$type_id][$name];
      if (isset($config_value)) {
        $form[$type_id]['#collapsed'] = FALSE;
      }
      $form[$type_id][$name] = array(
        '#type' => 'select',
        '#title' => $field['widget']['label'],
        '#options' => $options,
        '#default_value' => isset($config_value) ? $config_value : '',
        '#description' => $field_types[$field['type']]['label'],
      );

      // Handle to + from date fields
      if (in_array($field['type'], array('date', 'datestamp', 'datetime')) && !empty($field['todate'])) {
        $form[$type_id][$name]['#title'] .= ' ('. t('From date') .')';
        $config_value = $config[$type_id][$name .'-todate'];
        if (isset($config_value)) {
          $form[$type_id]['#collapsed'] = FALSE;
        }
        $form[$type_id][$name .'-todate'] = array(
          '#type' => 'select',
          '#title' => $field['widget']['label'] .' ('. t('To date') .')',
          '#options' => $options,
          '#default_value' => isset($config_value) ? $config_value  : '',
          '#description' => $field_types[$field['type']]['label'],
        );
      }

      // Handle location fields
      if ($field['type'] == 'location') {
        $form[$type_id][$name] = array(
          '#type' => 'fieldset',
          '#title' => $field['widget']['label'],
        );
        foreach ($field['location_settings']['form']['fields'] as $loc_name => $loc_field) {
          if ($loc_field['collect']) {
            $form[$type_id][$name][$loc_name] = array(
              '#type' => 'select',
              '#title' => drupal_ucfirst(str_replace('_', ' ', $loc_name)),
              '#options' => $options,
              '#default_value' => isset($config_value[$loc_name]) ? $config_value[$loc_name] : '',
            );
          }
        }
      }
    }

    if (!empty($flag_options)) {
      $form[$type_id]['salsa_objects_flags'] = array(
        '#type' => 'select',
        '#title' => t('Flag used in this content type for user to join an event'),
        '#options' => $flag_options,
        '#default_value' => isset($flags_config[$type_id]) ? $flags_config[$type_id] : '',
      );
    }
  }

  $form['buttons']['submit'] = array(
    '#name' => 'submit',
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['buttons']['reset'] = array(
    '#name' => 'reset',
    '#type' => 'submit',
    '#value' => t('Reset to defaults'),
  );

  return $form;
}

function salsa_objects_mappings_submit($form, &$form_state) {
  if ($form_state['clicked_button']['#name'] != 'reset') {
    $config = array();
    foreach ($form_state['values'] as $key => $type) {
      if (is_array($type)) {
        foreach ($type as $field => $value) {
          if ($value && ($field != 'salsa_objects_flags')) {
            if (is_array($value)) {
              // Remove empty array items
              foreach ($value as $vk => $vv) {
                if (empty($vv)) {
                  unset($value[$vk]);
                }
              }
            }
            if (!empty($value)) {
              $config[$key][$field] = $value;
            }
          }
        }
      }
    }
    if (module_exists('flag')) {
      foreach ($form_state['values'] as $key => $type) {
        if (is_array($type)) {
          if (!empty($type['salsa_objects_flags'])) {
            $flags[$key] = $type['salsa_objects_flags'];
          }
        }
      }
      variable_set('salsa_objects_flags', $flags);
    }
    variable_set('salsa_objects_mappings', $config);
    variable_set('salsa_objects_conflict', $form_state['values']['salsa_objects_conflict']);
  }
  else {
    variable_del('salsa_objects_flags');
    variable_del('salsa_objects_mappings');
    variable_del('salsa_objects_conflict');
  }
}

/**
 * Event table fields from Salsa.
 *
 * With data from
 * http://salsalabs.com/o/8001/p/salsa/website/public2/commons/dev/docs/schema?object=event
 */
function salsa_objects_table_fields() {
  return array(
    'event_KEY' => array(
      'type' => 'number_integer',
    ),
    'organization_KEY' => array(
      'type' => 'number_integer',
    ),
    'chapter_KEY' => array(
      'type' => 'number_integer',
    ),
    'national_event_KEY' => array(
      'type' => 'number_integer',
    ),
    'distributed_event_KEY' => array(
      'type' => 'number_integer',
    ),
    'supporter_KEY' => array(
      'type' => 'number_integer',
    ),
    'Reference_Name' => array(
      'type' => 'text',
    ),
    'Event_Name' => array(
      'type' => 'text',
    ),
    'Last_Modified' => array(
      'type' => 'date',
    ),
    'Date_Created' => array(
      'type' => 'date',
    ),
    'Description' => array(
      'type' => 'text',
    ),
    'Address' => array(
      'type' => 'text',
    ),
    'City' => array(
      'type' => 'text',
    ),
    'State' => array(
      'type' => 'text',
    ),
    'Zip' => array(
      'type' => 'text',
    ),
    'Country' => array(
      'type' => 'text',
    ),
    'Directions' => array(
      'type' => 'text',
    ),
    'Header' => array(
      'type' => 'text',
    ),
    'Footer' => array(
      'type' => 'text',
    ),
    'PRIVATE_Zip_Plus_4' => array(
      'type' => 'text',
    ),
    'Start' => array(
      'type' => 'date',
    ),
    'End' => array(
      'type' => 'date',
    ),
    'Deadline' => array(
      'type' => 'date',
    ),
    'Recurrence_Frequency' => array(
      'type' => 'text',
    ),
    'Recurrence_Interval' => array(
      'type' => 'text',
    ),
    'Contact_Email' => array(
      'type' => 'text',
    ),
    'No_Registration' => array(
      'type' => 'number_integer',
    ),
    'Guests_allowed' => array(
      'type' => 'number_integer',
    ),
    'Maximum_Attendees' => array(
      'type' => 'number_integer',
    ),
    'Maximum_Waiting_List_Size' => array(
      'type' => 'number_integer',
    ),
    'Hide_Standard_Map' => array(
      'type' => 'number_integer',
    ),
    'Map_URL' => array(
      'type' => 'text',
    ),
    'Status' => array(
      'type' => 'text',
    ),
    'This_Event_Costs_Money' => array(
      'type' => 'number_integer',
    ),
    'Ticket_Price' => array(
      'type' => 'number_float',
    ),
    'merchant_account_KEY' => array(
      'type' => 'number_integer',
    ),
    'Default_Tracking_Code' => array(
      'type' => 'text',
    ),
    'redirect_path' => array(
      'type' => 'text',
    ),
    'Request' => array(
      'type' => 'text',
    ),
    'Required' => array(
      'type' => 'text',
    ),
    'groups_KEYS' => array(
      'type' => 'text',
    ),
    'Automatically_add_to_Groups' => array(
      'type' => 'number_integer',
    ),
    'Display_to_Chapters' => array(
      'type' => 'number_integer',
    ),
    'Request_Additional_Attendees' => array(
      'type' => 'number_integer',
    ),
    'One_Column_Layout' => array(
      'type' => 'number_integer',
    ),
    'Reminder_Status' => array(
      'type' => 'text',
    ),
    'Reminder_Hours' => array(
      'type' => 'number_integer',
    ),
    'Latitude' => array(
      'type' => 'number_float',
    ),
    'Longitude' => array(
      'type' => 'number_float',
    ),
    'address_md5sum' => array(
      'type' => 'text',
    ),
    'Template' => array(
      'type' => 'text',
    ),
    'Allow_Guest_Signup' => array(
      'type' => 'number_integer',
    ),
    'Maximum_Donation_Amount' => array(
      'type' => 'number_float',
    ),
    'Confirmation_Text' => array(
      'type' => 'text',
    ),
    'supporter_picture_KEY' => array(
      'type' => 'number_integer',
    ),
    'supporter_my_donate_page_KEY' => array(
      'type' => 'number_integer',
    ),
    'Location_Common_Name' => array(
      'type' => 'text',
    ),
  );
}
