<?php
// $Id: salsa_supporter.admin.inc,v 1.1.2.8 2011/02/22 20:35:54 jcnventura Exp $

/**
 * @file
 * Contains the administrative functions of the salsa_supporter module.
 *
 * This file is included by the salsa_supporter module, and includes the
 * settings form.
 */

/**
 * Menu callback for the Salsa Supporter module settings form.
 *
 * @ingroup forms
 */
function salsa_supporter_mappings() {
  $config = variable_get('salsa_supporter_mappings', array());
  $custom_fields = salsa_supporter_custom_fields();

  $supporter_fields = salsa_supporter_table_fields();
  $options = array('' => '');
  $options[] = '-- Default fields --';
  foreach ($supporter_fields as $field => $value) {
    $options[$field] = drupal_ucfirst(str_replace('_', ' ', $field));
  }
  $options[] = '-- Custom fields --';
  $custom_fields = array_keys(salsa_supporter_custom_fields());
  foreach ($custom_fields as $field) {
    $options[$field] = drupal_ucfirst(str_replace('_', ' ', $field));
  }

  $form['salsa_supporter_conflict'] = array(
    '#type' => 'select',
    '#title' => t('In case of conflict'),
    '#options' => array(
      'drupal' => t('Drupal wins'),
      'salsa' => t('Salsa wins'),
    ),
    '#default_value' => variable_get('salsa_supporter_conflict', 'drupal'),
  );
  if (module_exists('content_profile')) {
    if (module_exists('content')) {
      $types = content_types();
      $field_types = _content_field_types();
    }
    else {
      $types = node_get_types();
      $field_types = array('text' => array('label' => 'Text'));
    }
    $form['content_profile'] = array(
      '#tree' => TRUE,
    );

    foreach ($types as $type_id => $type) {
      $type = (array) $type;
      if (is_content_profile($type['type'])) {
        $form['content_profile'][$type_id] = array(
          '#type' => 'fieldset',
          '#title' => $type['name'] .' ('. t('Content type') .')',
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );

        // Fields.
        $fields = array();
        if ($type['has_title']) {
          $fields['title']['widget']['label'] = $type['title_label'];
          $fields['title']['type'] = 'text';
        }
        if ($type['has_body']) {
          $fields['body_field']['widget']['label'] = $type['body_label'];
          $fields['body_field']['type'] = 'text';
        }
        // Add CCK's fields
        if (isset($type['fields'])) {
          $fields = $fields + $type['fields'];
        }

        foreach ($fields as $name => $field) {
          $config_value = $config['content_profile'][$type_id][$name];
          if (isset($config_value)) {
            $form['content_profile'][$type_id]['#collapsed'] = FALSE;
          }
          $form['content_profile'][$type_id][$name] = array(
            '#type' => 'select',
            '#title' => $field['widget']['label'],
            '#options' => $options,
            '#default_value' => isset($config_value) ? $config_value : '',
            '#description' => $field_types[$field['type']]['label'],
          );

          // Handle to + from date fields
          if (in_array($field['type'], array('date', 'datestamp', 'datetime')) && !empty($field['todate'])) {
            $form['content_profile'][$type_id][$name]['#title'] .= ' ('. t('From date') .')';
            $config_value = $config['content_profile'][$type_id][$name .'-todate'];
            if (isset($config_value)) {
              $form['content_profile'][$type_id]['#collapsed'] = FALSE;
            }
            $form['content_profile'][$type_id][$name .'-todate'] = array(
              '#type' => 'select',
              '#title' => $field['widget']['label'] .' ('. t('To date') .')',
              '#options' => $options,
              '#default_value' => isset($config_value) ? $config_value  : '',
              '#description' => $field_types[$field['type']]['label'],
            );
          }

          // Handle location fields
          if ($field['type'] == 'location') {
            $form['content_profile'][$type_id][$name] = array(
              '#type' => 'fieldset',
              '#title' => $field['widget']['label'],
            );
            foreach ($field['location_settings']['form']['fields'] as $loc_name => $loc_field) {
              if ($loc_field['collect']) {
                $form['content_profile'][$type_id][$name][$loc_name] = array(
                  '#type' => 'select',
                  '#title' => drupal_ucfirst(str_replace('_', ' ', $loc_name)),
                  '#options' => $options,
                  '#default_value' => isset($config_value[$loc_name]) ? $config_value[$loc_name] : '',
                );
              }
            }
          }
        }
      }
    }
  }

  if (module_exists('profile')) {
    $categories = profile_categories();

    $form['profile'] = array(
      '#tree' => TRUE,
    );

    foreach ($categories as $category) {
      $category_id = $category['title'];
      $form['profile'][$category_id] = array(
        '#type' => 'fieldset',
        '#title' => $category['name'] .' ('. t('User profile field category') .')',
        '#tree' => TRUE,
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      );

      // Profile fields
      $result = _profile_get_fields($category_id);
      while ($field = db_fetch_object($result)) {
        $config_value = $config['profile'][$category_id][$field->name];
        if (isset($config_value)) {
          $form['profile'][$category_id]['#collapsed'] = FALSE;
        }
        $form['profile'][$category_id][$field->name] = array(
          '#type' => 'select',
          '#title' => $field->title,
          '#options' => $options,
          '#default_value' => isset($config_value) ? $config_value : '',
        );
      }
    }
  }

  $form['buttons']['submit'] = array(
    '#name' => 'submit',
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['buttons']['reset'] = array(
    '#name' => 'reset',
    '#type' => 'submit',
    '#value' => t('Reset to defaults'),
  );

  return $form;
}

function salsa_supporter_mappings_submit($form, &$form_state) {
  if ($form_state['clicked_button']['#name'] != 'reset') {
    $config['content_profile'] = $form_state['values']['content_profile'];
    $config['profile'] = $form_state['values']['profile'];
    foreach ($config as $type => $cats) {
      if ($cats) {
        foreach ($cats as $profile => $fields) {
          foreach ($fields as $field => $value) {
            if (is_array($value)) {
              // Remove empty array items
              foreach ($value as $vk => $vv) {
                if (empty($vv)) {
                  unset($config[$type][$profile][$field][$vk]);
                }
              }
            }
            if (empty($config[$type][$profile][$field])) {
              unset($config[$type][$profile][$field]);
            }
          }
        }
      }
    }
    variable_set('salsa_supporter_mappings', $config);
    variable_set('salsa_supporter_conflict', $form_state['values']['salsa_supporter_conflict']);
  }
  else {
    variable_del('salsa_supporter_mappings');
    variable_del('salsa_supporter_conflict');
  }
}

/**
 * Supporter table fields from Salsa.
 *
 * With data from
 * http://salsalabs.com/o/8001/p/salsa/website/public2/commons/dev/docs/schema?object=supporter
 */
function salsa_supporter_table_fields() {
  return array(
    'supporter_KEY' => array(
      'type' => 'number_integer',
    ),
    'organization_KEY' => array(
      'type' => 'number_integer',
    ),
    'chapter_KEY' => array(
      'type' => 'number_integer',
    ),
    'Last_Modified' => array(
      'type' => 'date',
    ),
    'Date_Created' => array(
      'type' => 'date',
    ),
    'Title' => array(
      'type' => 'text',
    ),
    'First_Name' => array(
      'type' => 'text',
    ),
    'MI' => array(
      'type' => 'text',
    ),
    'Last_Name' => array(
      'type' => 'text',
    ),
    'Suffix' => array(
      'type' => 'text',
    ),
    'Email' => array(
      'type' => 'text',
    ),
    'Password' => array(
      'type' => 'text',
    ),
    'Receive_Email' => array(
      'type' => 'number_integer',
    ),
    'Email_Status' => array(
      'type' => 'text',
      'options' => array('N/A', 'Good', 'Soft Bounce', 'Hard Bounce'),
    ),
    'Email_Preference' => array(
      'type' => 'text',
      'options' => array('HTML Email', 'Text Email'),
    ),
    'Soft_Bounce_Count' => array(
      'type' => 'number_integer',
    ),
    'Hard_Bounce_Count' => array(
      'type' => 'number_integer',
    ),
    'Last_Bounce' => array(
      'type' => 'date',
    ),
    'Receive_Phone_Blasts' => array(
      'type' => 'number_integer',
    ),
    'Phone' => array(
      'type' => 'text',
    ),
    'Cell_Phone' => array(
      'type' => 'text',
    ),
    'Phone_Provider' => array(
      'type' => 'text',
    ),
    'Work_Phone' => array(
      'type' => 'text',
    ),
    'Pager' => array(
      'type' => 'text',
    ),
    'Home_Fax' => array(
      'type' => 'text',
    ),
    'Work_Fax' => array(
      'type' => 'text',
    ),
    'Street' => array(
      'type' => 'text',
    ),
    'Street_2' => array(
      'type' => 'text',
    ),
    'Street_3' => array(
      'type' => 'text',
    ),
    'City' => array(
      'type' => 'text',
    ),
    'State' => array(
      'type' => 'text',
    ),
    'Zip' => array(
      'type' => 'text',
    ),
    'PRIVATE_Zip_Plus_4' => array(
      'type' => 'text',
    ),
    'County' => array(
      'type' => 'text',
    ),
    'District' => array(
      'type' => 'text',
    ),
    'Country' => array(
      'type' => 'text',
    ),
    'Latitude' => array(
      'type' => 'number_float',
    ),
    'Longitude' => array(
      'type' => 'number_float',
    ),
    'Organization' => array(
      'type' => 'text',
    ),
    'Department' => array(
      'type' => 'text',
    ),
    'Occupation' => array(
      'type' => 'text',
    ),
    'Instant_Messenger_Service' => array(
      'type' => 'text',
      'options' => array('AOL', 'Yahoo', 'MSN', 'ICQ', 'Other'),
    ),
    'Instant_Messenger_Name' => array(
      'type' => 'text',
    ),
    'Web_Page' => array(
      'type' => 'text',
    ),
    'Alternative_Email' => array(
      'type' => 'text',
    ),
    'Other_Data_1' => array(
      'type' => 'text',
    ),
    'Other_Data_2' => array(
      'type' => 'text',
    ),
    'Other_Data_3' => array(
      'type' => 'text',
    ),
    'Notes' => array(
      'type' => 'text',
    ),
    'Source' => array(
      'type' => 'text',
    ),
    'Source_Details' => array(
      'type' => 'text',
    ),
    'Source_Tracking_Code' => array(
      'type' => 'text',
    ),
    'Tracking_Code' => array(
      'type' => 'text',
    ),
    'Status' => array(
      'type' => 'text',
      'options' => array('Active', 'Temporarily Inactive', 'Invisible', 'Inactive'),
    ),
    'uid' => array(
      'type' => 'text',
    ),
    'Timezone' => array(
      'type' => 'text',
    ),
    'Language_Code' => array(
      'type' => 'text',
    ),
  );
}

function salsa_supporter_custom_fields() {
  $ret = array();
  $custom_fields = salsa_get_objects('custom_field');

  if ($custom_fields) {
    foreach ($custom_fields->xpath('//item') as $field) {
      switch (drupal_substr((string) $field->READONLY_Database_Field, 0, 2)) {
        case 'VA': // varchar
          $type = 'text';
          break;
        case 'BL': // blob
          $type = 'text';
          break;
        case 'IN': // integer
          $type = 'number_integer';
          break;
        case 'BO': // boolean
          $type = 'number_integer';
          break;
        case 'FL': // float
          $type = 'number_float';
          break;
        case 'DA': // datetime
          $type = 'date';
          break;
        case 'RA': // radio
          $type = 'text';
          break;
        default: // unknown
          $type = '';
      }
      $name = (string)$field->Name;
      $options = (string) $field->PRIVATE_Options;

      $ret[$name]['type'] = $type;
      if ($options !== '') {
        $ret[$name]['options'] = explode(',', $options);
      }
    }
  }

  return $ret;
}
